@@ -4,8 +4,9 @@ test_that("AddSeqContext for GRCh37", {
   skip_if("" == system.file(package = "BSgenome.Hsapiens.1000genomes.hs37d5"))
   stopifnot(requireNamespace("BSgenome.Hsapiens.1000genomes.hs37d5"))
   load("testdata/test_AddSeqContext.Rdata")
-  list.of.vcf <- ReadAndSplitStrelkaSBSVCFs("testdata/Strelka.SBS.GRCh37.vcf")
-  sbs.vcf <- list.of.vcf$SBS.vcfs[[1]]
+  list.of.vcfs <- 
+    ReadAndSplitStrelkaSBSVCFs("testdata/Strelka-SBS-GRCh37/Strelka.SBS.GRCh37.vcf")
+  sbs.vcf <- list.of.vcfs$SBS.vcfs[[1]]
   df <- 
     AddSeqContext(sbs.vcf, 
                   ref.genome = 
@@ -21,8 +22,8 @@ test_that("AddSeqContext for GRCh38", {
   skip_if("" == system.file(package = "BSgenome.Hsapiens.UCSC.hg38"))
   stopifnot(requireNamespace("BSgenome.Hsapiens.UCSC.hg38"))
   load("testdata/test_AddSeqContext.Rdata")
-  list.of.vcf <- ReadAndSplitStrelkaSBSVCFs("testdata/Strelka.SBS.GRCh38.vcf")
-  sbs.vcf <- list.of.vcf$SBS.vcfs[[1]]
+  list.of.vcfs <- ReadAndSplitStrelkaSBSVCFs("testdata/Strelka.SBS.GRCh38.vcf")
+  sbs.vcf <- list.of.vcfs$SBS.vcfs[[1]]
   df3 <- AddSeqContext(sbs.vcf, 
                        ref.genome = 
                        BSgenome.Hsapiens.UCSC.hg38::BSgenome.Hsapiens.UCSC.hg38)
@@ -37,8 +38,8 @@ test_that("AddSeqContext for GRCm38", {
   skip_if("" == system.file(package = "BSgenome.Mmusculus.UCSC.mm10"))
   stopifnot(requireNamespace("BSgenome.Mmusculus.UCSC.mm10"))
   load("testdata/test_AddSeqContext.Rdata")
-  list.of.vcf <- ReadAndSplitStrelkaSBSVCFs("testdata/Strelka.SBS.GRCm38.vcf")
-  sbs.vcf <- list.of.vcf$SBS.vcfs[[1]]
+  list.of.vcfs <- ReadAndSplitStrelkaSBSVCFs("testdata/Strelka.SBS.GRCm38.vcf")
+  sbs.vcf <- list.of.vcfs$SBS.vcfs[[1]]
   df6 <- AddSeqContext(sbs.vcf, ref.genome 
                        = BSgenome.Mmusculus.UCSC.mm10::BSgenome.Mmusculus.UCSC.mm10)
   df7 <- AddSeqContext(sbs.vcf, ref.genome = "GRCm38")
